@@ -149,6 +149,14 @@ generic_func_find_close_bracket(char_u *start)
 	return NULL;
     }
 
+    if (VIM_ISWHITE(*(p + 1)) && *skipwhite(p + 1) == '(')
+    {
+	// white space not allowed between '>' and '('
+	semsg(_(e_no_white_space_allowed_after_str_str), ">", start);
+	return NULL;
+    }
+
+
     if (type_count == 0)
     {
 	semsg(_(e_empty_type_list_for_generic_function_str), start);
@@ -298,7 +306,7 @@ parse_generic_func_type_args(
 
 	// parse the type
 	type_arg = parse_type(&p, &gfatab->gfat_arg_types, NULL, cctx, TRUE);
-	if (type_arg == NULL)
+	if (type_arg == NULL || !valid_declaration_type(type_arg))
 	    return NULL;
 
 	char	*ret_free = NULL;
@@ -326,28 +334,35 @@ parse_generic_func_type_args(
 
 	p = skipwhite(p);
 
+	if (*p == NUL || *p == '>')
+	    break;
+
 	// after a type, expect ',' or '>'
-	if (*p != ',' && *p != '>')
+	if (*p != ',')
 	{
 	    semsg(_(e_missing_comma_in_generic_function_str), start);
 	    return NULL;
 	}
 
-	// if there's a comma, require whitespace after it and skip it
-	if (*p == ',')
+	if (*(p + 1) == NUL)
+	    break;
+
+	// Require whitespace after a comma and skip it
+	if (!VIM_ISWHITE(*(p + 1)))
 	{
-	    if (!VIM_ISWHITE(*(p + 1)))
-	    {
-		semsg(_(e_white_space_required_after_str_str), ",", p);
-		return NULL;
-	    }
-	    p++;
+	    semsg(_(e_white_space_required_after_str_str), ",", p);
+	    return NULL;
 	}
+	p++;
     }
 
     // ensure the list of types ends in a closing '>'
     if (*p != '>')
+    {
+	semsg(_(e_missing_closing_angle_bracket_in_generic_function_str),
+		func_name);
 	return NULL;
+    }
 
     // no whitespace allowed before '>'
     if (VIM_ISWHITE(*(p - 1)))
@@ -511,16 +526,6 @@ parse_generic_func_type_params(
 	if (name_exists)
 	    return NULL;
 
-	if (ga_grow(&gfatab->gfat_param_types, 1) == FAIL)
-	    return NULL;
-	type_T *gt =
-	    &((type_T *)gfatab->gfat_param_types.ga_data)[gfatab->gfat_param_types.ga_len];
-	gfatab->gfat_param_types.ga_len++;
-
-	CLEAR_POINTER(gt);
-	gt->tt_type = VAR_ANY;
-	gt->tt_flags = TTFLAG_GENERIC;
-
 	if (ga_grow(&gfatab->gfat_args, 1) == FAIL)
 	    return NULL;
 	generic_T *generic =
@@ -531,7 +536,7 @@ parse_generic_func_type_params(
 	if (generic->gt_name == NULL)
 	    return NULL;
 	vim_strncpy(generic->gt_name, name_start, name_len);
-	generic->gt_type = gt;
+	generic->gt_type = NULL;
 
 	if (VIM_ISWHITE(*p))
 	{
@@ -557,13 +562,32 @@ parse_generic_func_type_params(
     }
     if (*p != '>')
 	return NULL;
+    p++;
 
-    if (generic_func_args_table_size(gfatab) == 0)
+    int gfat_sz = generic_func_args_table_size(gfatab);
+
+    if (gfat_sz == 0)
     {
 	emsg_funcname(e_empty_type_list_for_generic_function_str, func_name);
 	return NULL;
     }
-    p++;
+
+    // set the generic parms to VAR_ANY type
+    if (ga_grow(&gfatab->gfat_param_types, gfat_sz) == FAIL)
+	return NULL;
+
+    gfatab->gfat_param_types.ga_len = gfat_sz;
+    for (int i = 0; i < generic_func_args_table_size(gfatab); i++)
+    {
+	type_T *gt = &((type_T *)gfatab->gfat_param_types.ga_data)[i];
+
+	CLEAR_POINTER(gt);
+	gt->tt_type = VAR_ANY;
+	gt->tt_flags = TTFLAG_GENERIC;
+
+	generic_T *generic = &((generic_T *)gfatab->gfat_args.ga_data)[i];
+	generic->gt_type = gt;
+    }
 
     return p;
 }
